@@ -5,7 +5,13 @@
  * Then, from within bash, enable -f ./mypid enable_mypid, where ./mypid
  * is the binary obtained from running make. Hereafter, `${MYPID}'
  * is a shell builtin variable.
+ *
+ * This defines an unload hook function that is called when the builtin is
+ * deleted with enable -d that will unbind the MYPID variable so future
+ * references to it do not attempt to access memory that is no longer part
+ * of this process's address space.
  */
+#include <config.h>
 
 #include <stdio.h>
 #include <errno.h>
@@ -17,17 +23,16 @@
 #define INIT_DYNAMIC_VAR(var, val, gfunc, afunc) \
   do \
     { SHELL_VAR *v = bind_variable (var, (val), 0); \
-      v->dynamic_value = gfunc; \
-      v->assign_func = afunc; \
+      if (v) \
+	{ \
+	  v->dynamic_value = gfunc; \
+	  v->assign_func = afunc; \
+	} \
     } \
   while (0)
 
 static SHELL_VAR *
-assign_mypid (
-     SHELL_VAR *self,
-     char *value,
-     arrayind_t unused,
-     char *key )
+assign_mypid (SHELL_VAR *self, char *value, arrayind_t unused, char *key)
 {
   return (self);
 }
@@ -56,6 +61,12 @@ enable_mypid_builtin(WORD_LIST *list)
   return 0;
 }
 
+void
+enable_mypid_builtin_unload (char *s)
+{
+  unbind_variable ("MYPID");
+}
+
 char const *enable_mypid_doc[] = {
   "Enable $MYPID.",
   "",
