@@ -6,7 +6,7 @@
  */
 
 /*
-   Copyright (C) 1999-2009 Free Software Foundation, Inc.
+   Copyright (C) 1999-2009, 2022 Free Software Foundation, Inc.
 
    This file is part of GNU Bash.
    Bash is free software: you can redistribute it and/or modify
@@ -34,8 +34,7 @@
 extern char **export_env;
 
 int
-printenv_builtin (list) 
-     WORD_LIST *list;
+printenv_builtin (WORD_LIST *list)
 {
   register char **envp;
   int opt;
