@@ -4,7 +4,7 @@
  */
 
 /*
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2020,2022 Free Software Foundation, Inc.
 
    This file is part of GNU Bash.
    Bash is free software: you can redistribute it and/or modify
@@ -39,8 +39,7 @@ extern pid_t dollar_dollar_pid;
 extern int last_command_exit_value;
 
 int
-push_builtin (list)
-     WORD_LIST *list;
+push_builtin (WORD_LIST *list)
 {
   pid_t pid;
   int xstatus, opt;
