@@ -1,6 +1,6 @@
 /* findcmd.h - functions from findcmd.c. */
 
-/* Copyright (C) 1997-2015,2020 Free Software Foundation, Inc.
+/* Copyright (C) 1997-2015,2020-2023 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -28,16 +28,17 @@
 #define CMDSRCH_STDPATH		0x02
 #define CMDSRCH_TEMPENV		0x04
 
-extern int file_status PARAMS((const char *));
-extern int executable_file PARAMS((const char *));
-extern int is_directory PARAMS((const char *));
-extern int executable_or_directory PARAMS((const char *));
-extern char *find_user_command PARAMS((const char *));
-extern char *find_in_path PARAMS((const char *, char *, int));
-extern char *find_path_file PARAMS((const char *));
-extern char *search_for_command PARAMS((const char *, int));
-extern char *user_command_matches PARAMS((const char *, int, int));
-extern void setup_exec_ignore PARAMS((char *));
+extern int file_status (const char *);
+extern int executable_file (const char *);
+extern int is_directory (const char *);
+extern int executable_or_directory (const char *);
+extern char *find_user_command (const char *);
+extern char *find_in_path (const char *, char *, int);
+extern char *find_path_file (const char *);
+extern char *path_value (const char *, int);
+extern char *search_for_command (const char *, int);
+extern char *user_command_matches (const char *, int, int);
+extern void setup_exec_ignore (const char *);
 
 extern int dot_found_in_search;
 
