@@ -1,6 +1,6 @@
 /* glob.c -- file-name wildcard pattern matching for Bash.
 
-   Copyright (C) 1985-2020 Free Software Foundation, Inc.
+   Copyright (C) 1985-2023 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne-Again SHell.
    
@@ -60,14 +60,6 @@
 #  define bcopy(s, d, n) ((void) memcpy ((d), (s), (n)))
 #endif /* !HAVE_BCOPY && !bcopy */
 
-#if !defined (NULL)
-#  if defined (__STDC__)
-#    define NULL ((void *) 0)
-#  else
-#    define NULL 0x0
-#  endif /* __STDC__ */
-#endif /* !NULL */
-
 #if !defined (FREE)
 #  define FREE(x)	if (x) free (x)
 #endif
@@ -84,11 +76,11 @@ struct globval
     char *name;
   };
 
-extern void throw_to_top_level PARAMS((void));
-extern int sh_eaccess PARAMS((const char *, int));
-extern char *sh_makepath PARAMS((const char *, const char *, int));
-extern int signal_is_pending PARAMS((int));
-extern void run_pending_traps PARAMS((void));
+extern void throw_to_top_level (void);
+extern int sh_eaccess (const char *, int);
+extern char *sh_makepath (const char *, const char *, int);
+extern int signal_is_pending (int);
+extern void run_pending_traps (void);
 
 extern int extended_glob;
 
@@ -103,7 +95,7 @@ int glob_ignore_case = 0;
 /* Global variable controlling whether globbing ever returns . or ..
    regardless of the pattern. If set to 1, no glob pattern will ever
    match `.' or `..'. Disabled by default. */
-int glob_always_skip_dot_and_dotdot = 0;
+int glob_always_skip_dot_and_dotdot = 1;
 
 /* Global variable to return to signify an error in globbing. */
 char *glob_error_return;
@@ -111,29 +103,29 @@ char *glob_error_return;
 static struct globval finddirs_error_return;
 
 /* Some forward declarations. */
-static int skipname PARAMS((char *, char *, int));
+static int skipname (char *, char *, int);
 #if HANDLE_MULTIBYTE
-static int mbskipname PARAMS((char *, char *, int));
+static int mbskipname (char *, char *, int);
 #endif
-void udequote_pathname PARAMS((char *));
+void udequote_pathname (char *);
 #if HANDLE_MULTIBYTE
-void wcdequote_pathname PARAMS((wchar_t *));
-static void wdequote_pathname PARAMS((char *));
+void wcdequote_pathname (wchar_t *);
+static void wdequote_pathname (char *);
+static void dequote_pathname (char *);
 #else
-#  define dequote_pathname udequote_pathname
+#  define dequote_pathname(p) udequote_pathname(p)
 #endif
-static void dequote_pathname PARAMS((char *));
-static int glob_testdir PARAMS((char *, int));
-static char **glob_dir_to_array PARAMS((char *, char **, int));
+static int glob_testdir (char *, int);
+static char **glob_dir_to_array (char *, char **, int);
 
 /* Make sure these names continue to agree with what's in smatch.c */
-extern char *glob_patscan PARAMS((char *, char *, int));
-extern wchar_t *glob_patscan_wc PARAMS((wchar_t *, wchar_t *, int));
+extern char *glob_patscan (char *, char *, int, int);
+extern wchar_t *glob_patscan_wc (wchar_t *, wchar_t *, wint_t, int);
 
 /* And this from gmisc.c/gm_loop.c */
-extern int wextglob_pattern_p PARAMS((wchar_t *));
+extern int wextglob_pattern_p (wchar_t *);
 
-extern char *glob_dirscan PARAMS((char *, int));
+extern char *glob_dirscan (char *, int);
 
 /* Compile `glob_loop.c' for single-byte characters. */
 #define GCHAR	unsigned char
@@ -158,8 +150,7 @@ extern char *glob_dirscan PARAMS((char *, int));
 /* And now a function that calls either the single-byte or multibyte version
    of internal_glob_pattern_p. */
 int
-glob_pattern_p (pattern)
-     const char *pattern;
+glob_pattern_p (const char *pattern)
 {
 #if HANDLE_MULTIBYTE
   size_t n;
@@ -185,39 +176,44 @@ glob_pattern_p (pattern)
 }
 
 #if EXTENDED_GLOB
+
+#if defined (HANDLE_MULTIBYTE)
+#  define XSKIPNAME(p, d, f)	mbskipname(p, d, f)
+#else
+#  define XSKIPNAME(p, d, f)	skipname(p, d, f)
+#endif
+
 /* Return 1 if all subpatterns in the extended globbing pattern PAT indicate
    that the name should be skipped.  XXX - doesn't handle pattern negation,
    not sure if it should */
 static int
-extglob_skipname (pat, dname, flags)
-     char *pat, *dname;
-     int flags;
+extglob_skipname (char *pat, char *dname, int flags)
 {
   char *pp, *pe, *t, *se;
-  int n, r, negate, wild, nullpat;
+  int n, r, negate, wild, nullpat, xflags;
 
   negate = *pat == '!';
   wild = *pat == '*' || *pat == '?';
   pp = pat + 2;
-  se = pp + strlen (pp) - 1;		/* end of string */
-  pe = glob_patscan (pp, se, 0);	/* end of extglob pattern (( */
-  /* we should check for invalid extglob pattern here */
+  se = pp + strlen (pp);		/* end of pattern string */
+  pe = glob_patscan (pp, se, 0, 0);	/* end of extglob pattern */
+
+  /* if pe == 0, this is an invalid extglob pattern */
   if (pe == 0)
     return 0;
 
+  xflags = flags | ( negate ? GX_NEGATE : 0);
+
   /* if pe != se we have more of the pattern at the end of the extglob
      pattern. Check the easy case first ( */
-  if (pe == se && *pe == ')' && (t = strchr (pp, '|')) == 0)
+  if (pe == se && *pe == 0 && pe[-1] == ')' && (t = strchr (pp, '|')) == 0)
     {
-      *pe = '\0';
-#if defined (HANDLE_MULTIBYTE)
-      r = mbskipname (pp, dname, flags);
-#else
-      r = skipname (pp, dname, flags);	/*(*/
-#endif
-      *pe = ')';
-      if (wild && pe[1])	/* if we can match zero instances, check further */
-        return (skipname (pe+1, dname, flags));
+      pe[-1] = '\0';
+      /* This is where we check whether the pattern is being negated and
+	 match all files beginning with `.' if the pattern begins with a
+	 literal `.'. */
+      r = XSKIPNAME (pp, dname, xflags); /*(*/
+      pe[-1] = ')';
       return r;
     }
 
@@ -227,23 +223,23 @@ extglob_skipname (pat, dname, flags)
   nullpat = pe >= (pat + 2) && pe[-2] == '(' && pe[-1] == ')';
 
   /* check every subpattern */
-  while (t = glob_patscan (pp, pe, '|'))
+  while (t = glob_patscan (pp, pe, '|', 0))
     {
+      /* If T == PE and *T == 0 (&& PE[-1] == RPAREN), we have hit the end
+	 of a pattern with no trailing characters. */
       n = t[-1];	/* ( */
-      if (extglob_pattern_p (pp) && n == ')')
+      if (extglob_pattern_p (pp) && n == ')')		/* nested extglob? */
 	t[-1] = n;	/* no-op for now */
       else
 	t[-1] = '\0';
-#if defined (HANDLE_MULTIBYTE)
-      r = mbskipname (pp, dname, flags);
-#else
-      r = skipname (pp, dname, flags);
-#endif
+      r = XSKIPNAME (pp, dname, xflags);
       t[-1] = n;
       if (r == 0)	/* if any pattern says not skip, we don't skip */
         return r;
       pp = t;
-    }	/*(*/
+      if (pp == pe)
+	break;
+    }
 
   /* glob_patscan might find end of string */
   if (pp == se)
@@ -251,7 +247,8 @@ extglob_skipname (pat, dname, flags)
 
   /* but if it doesn't then we didn't match a leading dot */
   if (wild && *pe)	/* if we can match zero instances, check further */
-    return (skipname (pe, dname, flags));
+    return (XSKIPNAME (pe, dname, flags));
+
   return 1;
 }
 #endif
@@ -259,10 +256,7 @@ extglob_skipname (pat, dname, flags)
 /* Return 1 if DNAME should be skipped according to PAT.  Mostly concerned
    with matching leading `.'. */
 static int
-skipname (pat, dname, flags)
-     char *pat;
-     char *dname;
-     int flags;
+skipname (char *pat, char *dname, int flags)
 {
 #if EXTENDED_GLOB
   if (extglob_pattern_p (pat))		/* XXX */
@@ -279,9 +273,19 @@ skipname (pat, dname, flags)
 	DOT_OR_DOTDOT (dname))
     return 1;
 
+#if 0
+  /* This is where we check whether the pattern is being negated and
+     match all files beginning with `.' if the pattern begins with a
+     literal `.'. This is the negation of the next clause. */
+  else if ((flags & GX_NEGATE) && noglob_dot_filenames == 0 &&
+	dname[0] == '.' &&
+	(pat[0] == '.' || (pat[0] == '\\' && pat[1] == '.')))
+    return 0;
+#endif
+
   /* If a dot must be explicitly matched, check to see if they do. */
-  else if (noglob_dot_filenames && dname[0] == '.' && pat[0] != '.' &&
-	(pat[0] != '\\' || pat[1] != '.'))
+  else if (noglob_dot_filenames && dname[0] == '.' &&
+ 	   pat[0] != '.' && (pat[0] != '\\' || pat[1] != '.'))
     return 1;
 
   return 0;
@@ -290,9 +294,7 @@ skipname (pat, dname, flags)
 #if HANDLE_MULTIBYTE
 
 static int
-wskipname (pat, dname, flags)
-     wchar_t *pat, *dname;
-     int flags;
+wskipname (wchar_t *pat, wchar_t *dname, int flags)
 {
   if (glob_always_skip_dot_and_dotdot && WDOT_OR_DOTDOT (dname))
     return 1;
@@ -304,38 +306,51 @@ wskipname (pat, dname, flags)
 	WDOT_OR_DOTDOT (dname))
     return 1;
 
+#if 0
+  /* This is where we check whether the pattern is being negated and
+     match all files beginning with `.' if the pattern begins with a
+     literal `.'. This is the negation of the next clause. */
+  else if ((flags & GX_NEGATE) && noglob_dot_filenames == 0 &&
+	dname[0] == L'.' &&
+	(pat[0] == L'.' || (pat[0] == L'\\' && pat[1] == L'.')))
+    return 0;
+#endif
+
   /* If a leading dot must be explicitly matched, check to see if the
      pattern and dirname both have one. */
- else if (noglob_dot_filenames && dname[0] == L'.' &&
-	pat[0] != L'.' &&
-	   (pat[0] != L'\\' || pat[1] != L'.'))
+  else if (noglob_dot_filenames && dname[0] == L'.' &&
+	pat[0] != L'.' && (pat[0] != L'\\' || pat[1] != L'.'))
     return 1;
 
   return 0;
 }
 
 static int
-wextglob_skipname (pat, dname, flags)
-     wchar_t *pat, *dname;
-     int flags;
+wextglob_skipname (wchar_t *pat, wchar_t *dname, int flags)
 {
 #if EXTENDED_GLOB
-  wchar_t *pp, *pe, *t, n, *se;
-  int r, negate, wild, nullpat;
+  wchar_t *pp, *pe, *t, *se, n;
+  int r, negate, wild, nullpat, xflags;
 
   negate = *pat == L'!';
   wild = *pat == L'*' || *pat == L'?';
   pp = pat + 2;
-  se = pp + wcslen (pp) - 1;	/*(*/
-  pe = glob_patscan_wc (pp, se, 0);
+  se = pp + wcslen (pp);
+  pe = glob_patscan_wc (pp, se, 0, 0);
 
-  if (pe == se && *pe == ')' && (t = wcschr (pp, L'|')) == 0)
+  /* if pe == 0, this is an invalid extglob pattern */
+  if (pe == 0)
+    return 0;
+
+  xflags = flags | ( negate ? GX_NEGATE : 0);
+
+  /* if pe != se we have more of the pattern at the end of the extglob
+     pattern. Check the easy case first ( */
+  if (pe == se && *pe == L'\0' && pe[-1] == L')' && (t = wcschr (pp, L'|')) == 0)
     {
-      *pe = L'\0';
-      r = wskipname (pp, dname, flags); /*(*/
-      *pe = L')';
-      if (wild && pe[1] != L'\0')
-        return (wskipname (pe+1, dname, flags));
+      pe[-1] = L'\0';
+      r = wskipname (pp, dname, xflags); /*(*/
+      pe[-1] = L')';
       return r;
     }
 
@@ -345,26 +360,30 @@ wextglob_skipname (pat, dname, flags)
   nullpat = pe >= (pat + 2) && pe[-2] == L'(' && pe[-1] == L')';
 
   /* check every subpattern */
-  while (t = glob_patscan_wc (pp, pe, '|'))
+  while (t = glob_patscan_wc (pp, pe, '|', 0))
     {
       n = t[-1];	/* ( */
-      if (wextglob_pattern_p (pp) && n == L')')
+      if (wextglob_pattern_p (pp) && n == L')')		/* nested extglob? */
 	t[-1] = n;	/* no-op for now */
       else
 	t[-1] = L'\0';
-      r = wskipname (pp, dname, flags);
+      r = wskipname (pp, dname, xflags);
       t[-1] = n;
       if (r == 0)
 	return 0;
       pp = t;
+      if (pp == pe)
+	break;
     }
 
-  if (pp == pe)		/* glob_patscan_wc might find end of pattern */
+  /* glob_patscan_wc might find end of string */
+  if (pp == se)
     return r;
 
   /* but if it doesn't then we didn't match a leading dot */
   if (wild && *pe != L'\0')
     return (wskipname (pe, dname, flags));
+
   return 1;
 #else
   return (wskipname (pat, dname, flags));
@@ -374,9 +393,7 @@ wextglob_skipname (pat, dname, flags)
 /* Return 1 if DNAME should be skipped according to PAT.  Handles multibyte
    characters in PAT and DNAME.  Mostly concerned with matching leading `.'. */
 static int
-mbskipname (pat, dname, flags)
-     char *pat, *dname;
-     int flags;
+mbskipname (char *pat, char *dname, int flags)
 {
   int ret, ext;
   wchar_t *pat_wc, *dn_wc;
@@ -411,8 +428,7 @@ mbskipname (pat, dname, flags)
 
 /* Remove backslashes quoting characters in PATHNAME by modifying PATHNAME. */
 void
-udequote_pathname (pathname)
-     char *pathname;
+udequote_pathname (char *pathname)
 {
   register int i, j;
 
@@ -433,8 +449,7 @@ udequote_pathname (pathname)
 #if HANDLE_MULTIBYTE
 /* Remove backslashes quoting characters in PATHNAME by modifying PATHNAME. */
 void
-wcdequote_pathname (wpathname)
-     wchar_t *wpathname;
+wcdequote_pathname (wchar_t *wpathname)
 {
   int i, j;
 
@@ -453,13 +468,11 @@ wcdequote_pathname (wpathname)
 }
 
 static void
-wdequote_pathname (pathname)
-     char *pathname;
+wdequote_pathname (char *pathname)
 {
   mbstate_t ps;
   size_t len, n;
   wchar_t *wpathname;
-  int i, j;
   wchar_t *orig_wpathname;
 
   if (mbsmbchar (pathname) == 0)
@@ -497,8 +510,7 @@ wdequote_pathname (pathname)
 }
 
 static void
-dequote_pathname (pathname)
-     char *pathname;
+dequote_pathname (char *pathname)
 {
   if (MB_CUR_MAX > 1)
     wdequote_pathname (pathname);
@@ -521,9 +533,7 @@ dequote_pathname (pathname)
 
 /* Return 0 if DIR is a directory, -2 if DIR is a symlink,  -1 otherwise. */
 static int
-glob_testdir (dir, flags)
-     char *dir;
-     int flags;
+glob_testdir (char *dir, int flags)
 {
   struct stat finfo;
   int r;
@@ -555,14 +565,9 @@ glob_testdir (dir, flags)
    directories in the returned list.  These two variables exist for the
    convenience of the caller (always glob_vector). */
 static struct globval *
-finddirs (pat, sdir, flags, ep, np)
-     char *pat;
-     char *sdir;
-     int flags;
-     struct globval **ep;
-     int *np;
+finddirs (char *pat, char *sdir, int flags, struct globval **ep, int *np)
 {
-  char **r, *n;
+  char **r;
   int ndirs;
   struct globval *ret, *e, *g;
 
@@ -633,29 +638,26 @@ finddirs (pat, sdir, flags, ep, np)
    Look in errno for more information.  */
 
 char **
-glob_vector (pat, dir, flags)
-     char *pat;
-     char *dir;
-     int flags;
+glob_vector (char *pat, char *dir, int flags)
 {
   DIR *d;
-  register struct dirent *dp;
-  struct globval *lastlink, *e, *dirlist;
-  register struct globval *nextlink;
-  register char *nextname, *npat, *subdir;
-  unsigned int count;
-  int lose, skip, ndirs, isdir, sdlen, add_current, patlen;
-  register char **name_vector;
-  register unsigned int i;
+  struct dirent *dp;
+  struct globval *nextlink, *lastlink, *e, *dirlist;
+  char *nextname, *npat, *subdir;
+  unsigned int count, i;
+  int lose, skip, ndirs, isdir, add_current;
+  size_t patlen, sdlen;
+  char **name_vector;
   int mflags;		/* Flags passed to strmatch (). */
   int pflags;		/* flags passed to sh_makepath () */
   int hasglob;		/* return value from glob_pattern_p */
   int nalloca;
   struct globval *firstmalloc, *tmplink;
-  char *convfn;
+  char *convfn, *convpat;
 
   lastlink = 0;
-  count = lose = skip = add_current = 0;
+  count = 0;
+  lose = skip = add_current = 0;
 
   firstmalloc = 0;
   nalloca = 0;
@@ -698,7 +700,7 @@ glob_vector (pat, dir, flags)
   hasglob = 0;
   if (skip == 0 && ((hasglob = glob_pattern_p (pat)) == 0 || hasglob == 2))
     {
-      int dirlen;
+      size_t dirlen;
       struct stat finfo;
 
       if (glob_testdir (dir, 0) < 0)
@@ -763,9 +765,13 @@ glob_vector (pat, dir, flags)
       if (d == NULL)
 	return ((char **) &glob_error_return);
 
+      convpat = fnx_fromfs (pat, patlen);
+      if (convpat != pat)
+	convpat = savestring (convpat);
+
       /* Compute the flags that will be passed to strmatch().  We don't
 	 need to do this every time through the loop. */
-      mflags = (noglob_dot_filenames ? FNM_PERIOD : 0) | FNM_PATHNAME;
+      mflags = (noglob_dot_filenames ? FNM_PERIOD : FNM_DOTDOT) | FNM_PATHNAME;
 
 #ifdef FNM_CASEFOLD
       if (glob_ignore_case)
@@ -777,8 +783,7 @@ glob_vector (pat, dir, flags)
 
       add_current = ((flags & (GX_ALLDIRS|GX_ADDCURDIR)) == (GX_ALLDIRS|GX_ADDCURDIR));
 
-      /* Scan the directory, finding all names that match.
-	 For each name that matches, allocate a struct globval
+      /* Scan the directory, finding all names that match	 For each name that matches, allocate a struct globval
 	 on the stack and store the name in it.
 	 Chain those structs together; lastlink is the front of the chain.  */
       while (1)
@@ -860,6 +865,9 @@ glob_vector (pat, dir, flags)
 	      nextname = (char *) malloc (sdlen + 1);
 	      if (nextlink == 0 || nextname == 0)
 		{
+		  if (firstmalloc && firstmalloc == nextlink)
+		    firstmalloc = 0;
+		  /* If we reset FIRSTMALLOC we can free this here. */
 		  FREE (nextlink);
 		  FREE (nextname);
 		  free (subdir);
@@ -878,7 +886,7 @@ glob_vector (pat, dir, flags)
 	    free (subdir);
 
 	  convfn = fnx_fromfs (dp->d_name, D_NAMLEN (dp));
-	  if (strmatch (pat, convfn, mflags) != FNM_NOMATCH)
+	  if (strmatch (convpat, convfn, mflags) != FNM_NOMATCH)
 	    {
 	      if (nalloca < ALLOCA_MAX)
 		{
@@ -895,7 +903,18 @@ glob_vector (pat, dir, flags)
 	      nextname = (char *) malloc (D_NAMLEN (dp) + 1);
 	      if (nextlink == 0 || nextname == 0)
 		{
-		  FREE (nextlink);
+		  /* We free NEXTLINK here, since it won't be added to the
+		     LASTLINK chain. If we used malloc, and it returned non-
+		     NULL, firstmalloc will be set to something valid. If it's
+		     NEXTLINK, reset it before we free NEXTLINK to avoid
+		     duplicate frees. If not, it will be taken care of by the
+		     loop below with TMPLINK. */
+		  if (firstmalloc)
+		    {
+		      if (firstmalloc == nextlink)
+			firstmalloc = 0;
+		      FREE (nextlink);
+		    }
 		  FREE (nextname);
 		  lose = 1;
 		  break;
@@ -909,12 +928,15 @@ glob_vector (pat, dir, flags)
 	}
 
       (void) closedir (d);
+
+      if (convpat != pat)
+	free (convpat);
     }
 
   /* compat: if GX_ADDCURDIR, add the passed directory also.  Add an empty
      directory name as a placeholder if GX_NULLDIR (in which case the passed
      directory name is "."). */
-  if (add_current)
+  if (add_current && lose == 0)
     {
       sdlen = strlen (dir);
       nextname = (char *)malloc (sdlen + 1);
@@ -944,7 +966,7 @@ glob_vector (pat, dir, flags)
       lose |= name_vector == NULL;
     }
 
-  /* Have we run out of memory?	 */
+  /* Have we run out of memory or been interrupted? */
   if (lose)
     {
       tmplink = 0;
@@ -1005,11 +1027,10 @@ glob_vector (pat, dir, flags)
    it takes care of free()ing that array.  Thus, you might think of this
    function as side-effecting ARRAY.  This should handle GX_MARKDIRS. */
 static char **
-glob_dir_to_array (dir, array, flags)
-     char *dir, **array;
-     int flags;
+glob_dir_to_array (char *dir, char **array, int flags)
 {
-  register unsigned int i, l;
+  unsigned int i;
+  size_t l;
   int add_slash;
   char **result, *new;
   struct stat sb;
@@ -1094,14 +1115,12 @@ glob_dir_to_array (dir, array, flags)
    If there isn't enough memory, then return NULL.
    If a file system error occurs, return -1; `errno' has the error code.  */
 char **
-glob_filename (pathname, flags)
-     char *pathname;
-     int flags;
+glob_filename (char *pathname, int flags)
 {
   char **result, **new_result;
   unsigned int result_size;
   char *directory_name, *filename, *dname, *fn;
-  unsigned int directory_len;
+  size_t directory_len;
   int free_dirname;			/* flag */
   int dflags, hasglob;
 
@@ -1160,7 +1179,7 @@ glob_filename (pathname, flags)
   if (directory_len > 0 && (hasglob = glob_pattern_p (directory_name)) == 1)
     {
       char **directories, *d, *p;
-      register unsigned int i;
+      unsigned int i;
       int all_starstar, last_starstar;
 
       all_starstar = last_starstar = 0;
@@ -1196,7 +1215,7 @@ glob_filename (pathname, flags)
 	 so we can compensate if filename is [star][star] */
       if ((flags & GX_GLOBSTAR) && all_starstar == 0)
 	{
-	  int dl, prev;
+	  size_t dl, prev;
 	  prev = dl = directory_len;
 	  while (dl >= 4 && d[dl - 1] == '/' &&
 			   d[dl - 2] == '*' &&
@@ -1282,8 +1301,6 @@ glob_filename (pathname, flags)
 	  /* Special handling for symlinks to directories with globstar on */
 	  if (all_starstar && (dflags & GX_NULLDIR) == 0)
 	    {
-	      int dlen;
-
 	      /* If we have a directory name that is not null (GX_NULLDIR above)
 		 and is a symlink to a directory, we return the symlink if
 		 we're not `descending' into it (filename[0] == 0) and return
@@ -1326,7 +1343,7 @@ glob_filename (pathname, flags)
 	  else
 	    {
 	      char **array;
-	      register unsigned int l;
+	      unsigned int l;
 
 	      /* If we're expanding **, we don't need to glue the directory
 		 name to the results; we've already done it in glob_vector */
@@ -1350,7 +1367,7 @@ glob_filename (pathname, flags)
 			NULL_PLACEHOLDER (temp_results))
 #undef NULL_PLACEHOLDER
 		    {
-		      register int i, n;
+		      int i, n;
 		      for (n = 0; temp_results[n] && *temp_results[n] == 0; n++)
 			;
 		      i = n;
@@ -1392,6 +1409,12 @@ glob_filename (pathname, flags)
 		free ((char *) array);
 	      else if ((dflags & GX_ALLDIRS) && filename[0] == '*' && filename[1] == '*' && filename[2] == '\0')
 		free (temp_results);	/* expanding ** case above */
+	      else if (array == temp_results)
+		/* If array == temp_results, either we assigned it above or
+		   glob_dir_to_array returned temp_results because the dirname
+		   was the empty string. In any case, we assume temp_results
+		   has not been freed, and free it here. */
+		free (temp_results);
 
 	      if (shouldbreak)
 		break;
@@ -1441,6 +1464,7 @@ only_filename:
 	    {
 	      if (free_dirname)
 		free (directory_name);
+	      free ((char *) result);
 	      return ((char **)&glob_error_return);
 	    }
 	}
@@ -1506,6 +1530,7 @@ only_filename:
 
       if (free_dirname)
 	free (directory_name);
+
       return (result);
     }
 
