@@ -1,6 +1,6 @@
 /* xmbsrtowcs.c -- replacement function for mbsrtowcs */
 
-/* Copyright (C) 2002-2010 Free Software Foundation, Inc.
+/* Copyright (C) 2002-2022 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -35,12 +35,19 @@
 
 #if HANDLE_MULTIBYTE
 
+#include <errno.h>
+#if !defined (errno)
+extern int errno;
+#endif
+
+#define WSBUF_INC 32
+
 #ifndef FREE
 #  define FREE(x)	do { if (x) free (x); } while (0)
 #endif
 
 #if ! HAVE_STRCHRNUL
-extern char *strchrnul __P((const char *, int));
+extern char *strchrnul (const char *, int);
 #endif
 
 /* On some locales (ex. ja_JP.sjis), mbsrtowc doesn't convert 0x5c to U<0x5c>.
@@ -50,11 +57,7 @@ static mbstate_t local_state;
 static int local_state_use = 0;
 
 size_t
-xmbsrtowcs (dest, src, len, pstate)
-    wchar_t *dest;
-    const char **src;
-    size_t len;
-    mbstate_t *pstate;
+xmbsrtowcs (wchar_t *dest, const char **src, size_t len, mbstate_t *pstate)
 {
   mbstate_t *ps;
   size_t mblength, wclength, n;
@@ -132,23 +135,22 @@ xmbsrtowcs (dest, src, len, pstate)
 }
 
 #if HAVE_MBSNRTOWCS
-/* Convert a multibyte string to a wide character string. Memory for the
-   new wide character string is obtained with malloc.
+/* Convert a multibyte string SRC to a wide character string and store the
+   result in *DESTP. Memory for the new wide character string is obtained
+   with malloc.
 
    Fast multiple-character version of xdupmbstowcs used when the indices are
    not required and mbsnrtowcs is available. */
 
 static size_t
-xdupmbstowcs2 (destp, src)
-    wchar_t **destp;	/* Store the pointer to the wide character string */
-    const char *src;	/* Multibyte character string */
+xdupmbstowcs2 (wchar_t **destp, const char *src)
 {
   const char *p;	/* Conversion start position of src */
   wchar_t *wsbuf;	/* Buffer for wide characters. */
   size_t wsbuf_size;	/* Size of WSBUF */
   size_t wcnum;		/* Number of wide characters in WSBUF */
   mbstate_t state;	/* Conversion State */
-  size_t wcslength;	/* Number of wide characters produced by the conversion. */
+  size_t n, wcslength;	/* Number of wide characters produced by the conversion. */
   const char *end_or_backslash;
   size_t nms;	/* Number of multibyte characters to convert at one time. */
   mbstate_t tmp_state;
@@ -164,14 +166,25 @@ xdupmbstowcs2 (destp, src)
   do
     {
       end_or_backslash = strchrnul(p, '\\');
-      nms = (end_or_backslash - p);
+      nms = end_or_backslash - p;
       if (*end_or_backslash == '\0')
 	nms++;
 
       /* Compute the number of produced wide-characters. */
       tmp_p = p;
       tmp_state = state;
-      wcslength = mbsnrtowcs(NULL, &tmp_p, nms, 0, &tmp_state);
+
+      if (nms == 0 && *p == '\\')	/* special initial case */
+	nms = wcslength = 1;
+      else
+	wcslength = mbsnrtowcs (NULL, &tmp_p, nms, 0, &tmp_state);
+
+      if (wcslength == 0)
+	{
+	  tmp_p = p;		/* will need below */
+	  tmp_state = state;
+	  wcslength = 1;	/* take a single byte */
+	}
 
       /* Conversion failed. */
       if (wcslength == (size_t)-1)
@@ -186,7 +199,8 @@ xdupmbstowcs2 (destp, src)
 	{
 	  wchar_t *wstmp;
 
-	  wsbuf_size = wcnum+wcslength+1;	/* 1 for the L'\0' or the potential L'\\' */
+	  while (wsbuf_size < wcnum+wcslength+1) /* 1 for the L'\0' or the potential L'\\' */
+	    wsbuf_size += WSBUF_INC;
 
 	  wstmp = (wchar_t *) realloc (wsbuf, wsbuf_size * sizeof (wchar_t));
 	  if (wstmp == NULL)
@@ -199,10 +213,30 @@ xdupmbstowcs2 (destp, src)
 	}
 
       /* Perform the conversion. This is assumed to return 'wcslength'.
-       * It may set 'p' to NULL. */
-      mbsnrtowcs(wsbuf+wcnum, &p, nms, wsbuf_size-wcnum, &state);
+	 It may set 'p' to NULL. */
+      n = mbsnrtowcs(wsbuf+wcnum, &p, nms, wsbuf_size-wcnum, &state);
 
-      wcnum += wcslength;
+      if (n == 0 && p == 0)
+	{
+	  wsbuf[wcnum] = L'\0';
+	  break;
+	}
+
+      /* Compensate for taking single byte on wcs conversion failure above. */
+      if (wcslength == 1 && (n == 0 || n == (size_t)-1))
+	{
+	  state = tmp_state;
+	  p = tmp_p;
+	  wsbuf[wcnum] = *p;
+	  if (*p == 0)
+	    break;
+	  else
+	    {
+	      wcnum++; p++;
+	    }
+	}
+      else
+        wcnum += wcslength;
 
       if (mbsinit (&state) && (p != NULL) && (*p == '\\'))
 	{
@@ -219,24 +253,20 @@ xdupmbstowcs2 (destp, src)
 }
 #endif /* HAVE_MBSNRTOWCS */
 
-/* Convert a multibyte string to a wide character string. Memory for the
-   new wide character string is obtained with malloc.
+/* Convert a multibyte string SRC to a wide character string and store the
+   result in *DESTP. Memory for the new wide character string is obtained
+   with malloc.
 
    The return value is the length of the wide character string. Returns a
-   pointer to the wide character string in DESTP. If INDICESP is not NULL,
+   pointer to the wide character string in *DESTP. If INDICESP is not NULL,
    INDICESP stores the pointer to the pointer array. Each pointer is to
    the first byte of each multibyte character. Memory for the pointer array
    is obtained with malloc, too.
    If conversion is failed, the return value is (size_t)-1 and the values
    of DESTP and INDICESP are NULL. */
 
-#define WSBUF_INC 32
-
 size_t
-xdupmbstowcs (destp, indicesp, src)
-    wchar_t **destp;	/* Store the pointer to the wide character string */
-    char ***indicesp;	/* Store the pointer to the pointer array. */
-    const char *src;	/* Multibyte character string */
+xdupmbstowcs (wchar_t **destp, char ***indicesp, const char *src)
 {
   const char *p;	/* Conversion start position of src */
   wchar_t wc;		/* Created wide character by conversion */
@@ -251,6 +281,8 @@ xdupmbstowcs (destp, indicesp, src)
     {
       if (destp)
 	*destp = NULL;
+      if (indicesp)
+	*indicesp = NULL;
       return (size_t)-1;
     }
 
@@ -266,6 +298,8 @@ xdupmbstowcs (destp, indicesp, src)
   if (wsbuf == NULL)
     {
       *destp = NULL;
+      if (indicesp)
+        *indicesp = NULL;
       return (size_t)-1;
     }
 
@@ -277,6 +311,7 @@ xdupmbstowcs (destp, indicesp, src)
 	{
 	  free (wsbuf);
 	  *destp = NULL;
+	  *indicesp = NULL;
 	  return (size_t)-1;
 	}
     }
@@ -311,6 +346,8 @@ xdupmbstowcs (destp, indicesp, src)
 	  free (wsbuf);
 	  FREE (indices);
 	  *destp = NULL;
+	  if (indicesp)
+	    *indicesp = NULL;
 	  return (size_t)-1;
 	}
 
@@ -330,18 +367,22 @@ xdupmbstowcs (destp, indicesp, src)
 	      free (wsbuf);
 	      FREE (indices);
 	      *destp = NULL;
+	      if (indicesp)
+		*indicesp = NULL;
 	      return (size_t)-1;
 	    }
 	  wsbuf = wstmp;
 
 	  if (indicesp)
 	    {
-	      idxtmp = (char **) realloc (indices, wsbuf_size * sizeof (char **));
+	      idxtmp = (char **) realloc (indices, wsbuf_size * sizeof (char *));
 	      if (idxtmp == NULL)
 		{
 		  free (wsbuf);
 		  free (indices);
 		  *destp = NULL;
+		  if (indicesp)
+		    *indicesp = NULL;
 		  return (size_t)-1;
 		}
 	      indices = idxtmp;
@@ -363,4 +404,112 @@ xdupmbstowcs (destp, indicesp, src)
   return (wcnum - 1);
 }
 
+/* Convert wide character string to multibyte character string. Treat invalid
+   wide characters as bytes.  Used only in unusual circumstances.
+
+   Written by Bruno Haible <bruno@clisp.org>, 2008, adapted by Chet Ramey
+   for use in Bash. */
+
+/* Convert wide character string *SRCP to a multibyte character string and
+   store the result in DEST. Store at most LEN bytes in DEST. */
+size_t
+xwcsrtombs (char *dest, const wchar_t **srcp, size_t len, mbstate_t *ps)
+{
+  const wchar_t *src;
+  size_t cur_max;			/* XXX - locale_cur_max */
+  char buf[64], *destptr, *tmp_dest;
+  unsigned char uc;
+
+  cur_max = MB_CUR_MAX;
+  if (cur_max > sizeof (buf))		/* Holy cow. */
+    return (size_t)-1;
+
+  src = *srcp;
+
+  if (dest != NULL)
+    {
+      destptr = dest;
+
+      for (; len > 0; src++)
+	{
+	  wchar_t wc;
+	  size_t ret;
+
+	  wc = *src;
+	  /* If we have room, store directly into DEST. */
+	  tmp_dest = destptr;
+	  ret = wcrtomb (len >= cur_max ? destptr : buf, wc, ps);
+
+	  if (ret == (size_t)(-1))		/* XXX */
+	    {
+	      /* Since this is used for globbing and other uses of filenames,
+		 treat invalid wide character sequences as bytes.  This is
+		 intended to be symmetric with xdupmbstowcs2. */
+handle_byte:
+	      destptr = tmp_dest;	/* in case wcrtomb modified it */
+	      uc = wc;
+	      ret = 1;
+	      if (len >= cur_max)
+		*destptr = uc;
+	      else
+		buf[0] = uc;
+	      if (ps)
+		memset (ps, 0, sizeof (mbstate_t));
+	    }
+
+	  if (ret > cur_max)		/* Holy cow */
+	    goto bad_input;
+
+	  if (len < ret)
+	    break;
+
+	  if (len < cur_max)
+	    memcpy (destptr, buf, ret);
+
+	  if (wc == 0)
+	    {
+	      src = NULL;
+	      /* Here mbsinit (ps).  */
+	      break;
+	    }
+	  destptr += ret;
+	  len -= ret;
+	}
+      *srcp = src;
+      return destptr - dest;
+    }
+  else
+    {
+      /* Ignore dest and len, don't store *srcp at the end, and
+	 don't clobber *ps.  */
+      mbstate_t state = *ps;
+      size_t totalcount = 0;
+
+      for (;; src++)
+	{
+	  wchar_t wc;
+	  size_t ret;
+
+	  wc = *src;
+	  ret = wcrtomb (buf, wc, &state);
+
+	  if (ret == (size_t)(-1))
+	    goto bad_input2;
+	  if (wc == 0)
+	    {
+	      /* Here mbsinit (&state).  */
+	      break;
+	    }
+	  totalcount += ret;
+	}
+      return totalcount;
+    }
+
+bad_input:
+  *srcp = src;
+bad_input2:
+  errno = EILSEQ;
+  return (size_t)(-1);
+}
+
 #endif /* HANDLE_MULTIBYTE */
