@@ -1,6 +1,6 @@
 /* malloc.c - dynamic memory allocation for bash. */
 
-/*  Copyright (C) 1985-2020 Free Software Foundation, Inc.
+/*  Copyright (C) 1985-2025 Free Software Foundation, Inc.
 
     This file is part of GNU Bash, the Bourne-Again SHell.
     
@@ -47,10 +47,10 @@
  */
 
 /*
- * nextf[i] is the pointer to the next free block of size 2^(i+3).  The
- * smallest allocatable block is 8 bytes.  The overhead information will
- * go in the first int of the block, and the returned pointer will point
- * to the second.
+ * nextf[i] is the pointer to the next free block of size 2^(i+5).  The
+ * smallest allocatable block is 32 bytes. The overhead information will
+ * go in the first 16 bytes of the block, and the returned pointer will point
+ * to the rest.
  */
 
 /* Define MEMSCRAMBLE to have free() write 0xcf into memory as it's freed, to
@@ -121,8 +121,8 @@
 #  define NO_VALLOC
 #endif
 
-/* SIZEOF_LONG * 4 - 2, usable bins from 1..NBUCKETS-1 */
-#define NBUCKETS	30
+#define MALLOC_PAGESIZE_MIN	4096
+#define MALLOC_INCR_PAGES	8192
 
 #define ISALLOC ((char) 0xf7)	/* magic byte that implies allocation */
 #define ISFREE ((char) 0x54)	/* magic byte that implies free block */
@@ -140,20 +140,14 @@
    enough room in the block for the new size.  Range checking is always
    done. */
 union mhead {
-#if SIZEOF_CHAR_P == 8
   bits64_t mh_align[2];						/* 16 */
-#else
-  bits64_t mh_align;						/* 8 */
-#endif
   struct {
     char mi_alloc; 		/* ISALLOC or ISFREE */		/* 1 */
     char mi_index;		/* index in nextf[] */		/* 1 */
     /* Remainder are valid only when block is allocated */
     u_bits16_t mi_magic2;	/* should be == MAGIC2 */	/* 2 */
     u_bits32_t mi_nbytes;	/* # of bytes allocated */	/* 4 */
-#if SIZEOF_CHAR_P == 8
     char mi_magic8[8];		/* MAGIC1 guard bytes */	/* 8 */
-#endif
   } minfo;
 };
 #define mh_alloc	minfo.mi_alloc
@@ -162,14 +156,14 @@ union mhead {
 #define mh_magic2	minfo.mi_magic2
 #define mh_magic8	minfo.mi_magic8
 
+#define MAGIC8_NUMBYTES	8
+#define MALLOC_SIZE_T		u_bits32_t
+
 #define MOVERHEAD	sizeof(union mhead)
 
-#if SIZEOF_CHAR_P == 8
-#define MALIGN_MASK	15
-#else
-#define MALIGN_MASK	7	/* one less than desired alignment */
-#endif
+#define MALIGN_MASK	15		/* one less than desired alignment */
 
+/* Guard bytes we write at the end of the allocation, encoding the size. */
 typedef union _malloc_guard {
   char s[4];
   u_bits32_t i;
@@ -181,6 +175,8 @@ typedef union _malloc_guard {
    because we want sizeof (union mhead)
    to describe the overhead for when the block is in use,
    and we do not want the free-list pointer to count in that.  */
+/* If we have mmap, this is not used for chunks larger than mmap_threshold,
+   since we munmap immediately on free(). */
 
 /* If SIZEOF_CHAR_P == 8, this goes into the mh_magic8 buffer at the end of
    the rest of the struct. This may need adjusting. */
@@ -194,10 +190,11 @@ typedef union _malloc_guard {
 /* Written in the bytes before the block's real space (-SIZEOF_CHAR_P bytes) */
 #define MAGIC1 0x55
 #define MAGIC2 0x5555
-#define MSLOP  4		/* 4 bytes extra for u_bits32_t size */
+
+#define MSLOP  4		/* 4 bytes extra for u_bits32_t end guard size */
 
 /* How many bytes are actually allocated for a request of size N --
-   rounded up to nearest multiple of 2*SIZEOF_CHAR_P after accounting for
+   rounded up to nearest multiple of 16 (alignment) after accounting for
    malloc overhead. */
 #define ALLOCATED_BYTES(n) \
 	(((n) + MOVERHEAD + MSLOP + MALIGN_MASK) & ~MALIGN_MASK)
@@ -211,18 +208,22 @@ typedef union _malloc_guard {
 
 /* Minimum and maximum bucket indices for block splitting (and to bound
    the search for a block to split). */
-#define SPLIT_MIN	2	/* XXX - was 3 */
-#define SPLIT_MID	11
-#define SPLIT_MAX	14
+#define SPLIT_MIN	1		/* 64 */
+#define SPLIT_MID	9		/* 16384 */
+#define SPLIT_MAX	12		/* 131072 */
 
 /* Minimum and maximum bucket indices for block coalescing. */
-#define COMBINE_MIN	2
-#define COMBINE_MAX	(pagebucket - 1)	/* XXX */
+#define COMBINE_MIN	1		/* 64 */
+#define COMBINE_MAX	(pagebucket - 1)	/* 2048 for 4096-byte pages */
 
-#define LESSCORE_MIN	10
-#define LESSCORE_FRC	13
+#define LESSCORE_MIN	8		/* 8192 */
+#define LESSCORE_FRC	11		/* 65536 */
 
-#define STARTBUCK	1
+/* Which bin do we prepopulate with the initial sbrk memory? */
+#define PREPOP_BIN	1
+#define PREPOP_SIZE	64
+
+#define STARTBUCK	0
 
 /* Should we use mmap for large allocations? */
 #if defined (HAVE_MMAP)
@@ -232,15 +233,25 @@ typedef union _malloc_guard {
 #endif
 
 #if defined (HAVE_MMAP) && defined (MAP_ANONYMOUS)
-#  define USE_MMAP
+#  define USE_MMAP	1
 #endif
 
 #if defined (USE_MMAP)
-#  define MMAP_THRESHOLD	14	/* must be >= SPLIT_MAX, COMBINE_MAX */
+#  define MMAP_THRESHOLD	12	/* must be >= SPLIT_MAX, COMBINE_MAX */
 #else
 #  define MMAP_THRESHOLD	(8 * SIZEOF_LONG)
 #endif
 
+/* We don't try to decipher the differences between the Linux-style and
+   BSD-style implementations of mremap here; we use the Linux one. */
+#if USE_MMAP == 1 && defined (HAVE_MREMAP) && defined (MREMAP_MAYMOVE)
+#  define USE_MREMAP 1
+#endif
+
+/* usable bins from STARTBUCK..NBUCKETS-1 */
+
+#define NBUCKETS	28
+
 /* Flags for the internal functions. */
 #define MALLOC_WRAPPER	0x01	/* wrapper function */
 #define MALLOC_INTERNAL	0x02	/* internal function calling another */
@@ -263,9 +274,9 @@ typedef union _malloc_guard {
 
 /* Use this when we want to be sure that NB is in bucket NU. */
 #define RIGHT_BUCKET(nb, nu) \
-	(((nb) > binsizes[(nu)-1]) && ((nb) <= binsizes[(nu)]))
+  (((nb) > (((nu) >= 1) ? binsizes[(nu)-1] : 0UL)) && ((nb) <= binsizes[(nu)]))
 
-/* nextf[i] is free list of blocks of size 2**(i + 3)  */
+/* nextf[i] is free list of blocks of size 2**(i + 5)  */
 
 static union mhead *nextf[NBUCKETS];
 
@@ -280,47 +291,50 @@ static int maxbuck;	/* highest bucket receiving allocation request. */
 static char *memtop;	/* top of heap */
 
 static const unsigned long binsizes[NBUCKETS] = {
-	8UL, 16UL, 32UL, 64UL, 128UL, 256UL, 512UL, 1024UL, 2048UL, 4096UL,
+	32UL, 64UL, 128UL, 256UL, 512UL, 1024UL, 2048UL, 4096UL,
 	8192UL, 16384UL, 32768UL, 65536UL, 131072UL, 262144UL, 524288UL,
 	1048576UL, 2097152UL, 4194304UL, 8388608UL, 16777216UL, 33554432UL,
 	67108864UL, 134217728UL, 268435456UL, 536870912UL, 1073741824UL,
 	2147483648UL, 4294967295UL
 };
 
-/* binsizes[x] == (1 << ((x) + 3)) */
-#define binsize(x)	binsizes[(x)]
+/* binsizes[x] == (1 << ((x) + 5)) */
+#define binsize(x)	binsizes[(size_t) (x)]
+
+#define MAXALLOC_SIZE	binsizes[NBUCKETS-1]
 
 #if !defined (errno)
 extern int errno;
 #endif
 
 /* Declarations for internal functions */
-static PTR_T internal_malloc PARAMS((size_t, const char *, int, int));
-static PTR_T internal_realloc PARAMS((PTR_T, size_t, const char *, int, int));
-static void internal_free PARAMS((PTR_T, const char *, int, int));
-static PTR_T internal_memalign PARAMS((size_t, size_t, const char *, int, int));
+static PTR_T internal_malloc (size_t, const char *, int, int);
+static PTR_T internal_realloc (PTR_T, size_t, const char *, int, int);
+static void internal_free (PTR_T, const char *, int, int);
+static PTR_T internal_memalign (size_t, size_t, const char *, int, int);
 #ifndef NO_CALLOC
-static PTR_T internal_calloc PARAMS((size_t, size_t, const char *, int, int));
-static void internal_cfree PARAMS((PTR_T, const char *, int, int));
+static PTR_T internal_calloc (size_t, size_t, const char *, int, int);
+static void internal_cfree (PTR_T, const char *, int, int);
 #endif
 #ifndef NO_VALLOC
-static PTR_T internal_valloc PARAMS((size_t, const char *, int, int));
+static PTR_T internal_valloc (size_t, const char *, int, int);
 #endif
+static PTR_T internal_remap (PTR_T, size_t, int, int);
 
 #if defined (botch)
-extern void botch ();
+extern void botch (const char *, ...);
 #else
-static void botch PARAMS((const char *, const char *, int));
+static void botch (const char *, const char *, int);
 #endif
-static void xbotch PARAMS((PTR_T, int, const char *, const char *, int));
+static void xbotch (PTR_T, int, const char *, const char *, int);
 
 #if !HAVE_DECL_SBRK
-extern char *sbrk ();
+extern char *sbrk (int);
 #endif /* !HAVE_DECL_SBRK */
 
 #ifdef SHELL
 extern int running_trap;
-extern int signal_is_trapped PARAMS((int));
+extern int signal_is_trapped (int);
 #endif
 
 #ifdef MALLOC_STATS
@@ -339,16 +353,13 @@ int malloc_mmap_threshold = MMAP_THRESHOLD;
 char _malloc_trace_buckets[NBUCKETS];
 
 /* These should really go into a header file. */
-extern void mtrace_alloc PARAMS((const char *, PTR_T, size_t, const char *, int));
-extern void mtrace_free PARAMS((PTR_T, int, const char *, int));
+extern void mtrace_alloc (const char *, PTR_T, size_t, const char *, int);
+extern void mtrace_free (PTR_T, int, const char *, int);
 #endif
 
 #if !defined (botch)
 static void
-botch (s, file, line)
-     const char *s;
-     const char *file;
-     int line;
+botch (const char *s, const char *file, int line)
 {
   fprintf (stderr, _("malloc: failed assertion: %s\n"), s);
   (void)fflush (stderr);
@@ -359,12 +370,7 @@ botch (s, file, line)
 /* print the file and line number that caused the assertion failure and
    call botch() to do whatever the application wants with the information */
 static void
-xbotch (mem, e, s, file, line)
-     PTR_T mem;
-     int e;
-     const char *s;
-     const char *file;
-     int line;
+xbotch (PTR_T mem, int e, const char *s, const char *file, int line)
 {
   fprintf (stderr, _("\r\nmalloc: %s:%d: assertion botched\r\n"),
 			file ? file : _("unknown"), line);
@@ -381,8 +387,7 @@ xbotch (mem, e, s, file, line)
    assumed to not be busy; the caller (morecore()) checks for this. 
    BUSY[NU] must be set to 1. */
 static void
-bcoalesce (nu)
-     register int nu;
+bcoalesce (int nu)
 {
   register union mhead *mp, *mp1, *mp2;
   register int nbuck;
@@ -450,8 +455,7 @@ bcoalesce (nu)
    is assumed to be empty.  Must be called with signals blocked (e.g.,
    by morecore()).  BUSY[NU] must be set to 1. */
 static void
-bsplit (nu)
-     register int nu;
+bsplit (int nu)
 {
   register union mhead *mp;
   int nbuck, nblks, split_max;
@@ -515,9 +519,7 @@ bsplit (nu)
 /* Take the memory block MP and add it to a chain < NU.  NU is the right bucket,
    but is busy.  This avoids memory orphaning. */
 static void
-xsplit (mp, nu)
-     union mhead *mp;
-     int nu;
+xsplit (union mhead *mp, int nu)
 {
   union mhead *nh;
   int nbuck, nblks, split_max;
@@ -556,8 +558,7 @@ xsplit (mp, nu)
 }
 
 void
-_malloc_block_signals (setp, osetp)
-     sigset_t *setp, *osetp;
+_malloc_block_signals (sigset_t *setp, sigset_t *osetp)
 {
 #ifdef HAVE_POSIX_SIGNALS
   sigfillset (setp);
@@ -571,8 +572,7 @@ _malloc_block_signals (setp, osetp)
 }
 
 void
-_malloc_unblock_signals (setp, osetp)
-     sigset_t *setp, *osetp;
+_malloc_unblock_signals (sigset_t *setp, sigset_t *osetp)
 {
 #ifdef HAVE_POSIX_SIGNALS
   sigprocmask (SIG_SETMASK, osetp, (sigset_t *)NULL);
@@ -583,12 +583,12 @@ _malloc_unblock_signals (setp, osetp)
 #endif
 }
 
+#if defined (USE_LESSCORE)
 /* Return some memory to the system by reducing the break.  This is only
    called with NU > pagebucket, so we're always assured of giving back
-   more than one page of memory. */  
+   more than one page of memory. NU is the size index we're discarding */ 
 static void
-lesscore (nu)			/* give system back some memory */
-     register int nu;		/* size index we're discarding  */
+lesscore (int nu)
 {
   long siz;
 
@@ -603,11 +603,11 @@ lesscore (nu)			/* give system back some memory */
   _mstats.nlesscore[nu]++;
 #endif
 }
+#endif /* USE_LESSCORE */
 
 /* Ask system for more memory; add to NEXTF[NU].  BUSY[NU] must be set to 1. */  
 static void
-morecore (nu)
-     register int nu;		/* size index to get more of  */
+morecore (int nu)
 {
   register union mhead *mp;
   register int nblks;
@@ -616,7 +616,7 @@ morecore (nu)
   sigset_t set, oset;
   int blocked_sigs;
 
-  /* Block all signals in case we are executed from a signal handler. */
+  /* Block signals in case we are executed from a signal handler. */
   blocked_sigs = 0;
 #ifdef SHELL
 #  if defined (SIGCHLD)
@@ -736,21 +736,13 @@ morecore_done:
 }
 
 static void
-malloc_debug_dummy ()
+malloc_debug_dummy (void)
 {
   write (1, "malloc_debug_dummy\n", 19);
 }
 
-#if SIZEOF_CHAR_P == 8
-#define PREPOP_BIN	3
-#define PREPOP_SIZE	64
-#else
-#define PREPOP_BIN	2
-#define PREPOP_SIZE	32
-#endif
-
 static int
-pagealign ()
+pagealign (void)
 {
   register int nunits;
   register union mhead *mp;
@@ -758,8 +750,8 @@ pagealign ()
   char *curbrk;
 
   pagesz = getpagesize ();
-  if (pagesz < 1024)
-    pagesz = 1024;
+  if (pagesz < MALLOC_PAGESIZE_MIN)
+    pagesz = MALLOC_PAGESIZE_MIN;
 
   /* OK, how much do we need to allocate to make things page-aligned?
      Some of this partial page will be wasted space, but we'll use as
@@ -815,17 +807,15 @@ pagealign ()
 
   return 0;
 }
-    
+
+/* allocate a block of memory */    
 static PTR_T
-internal_malloc (n, file, line, flags)		/* get a block */
-     size_t n;
-     const char *file;
-     int line, flags;
+internal_malloc (size_t n, const char *file, int line, int flags)
 {
   register union mhead *p;
   register int nunits;
   register char *m, *z;
-  long nbytes;
+  MALLOC_SIZE_T nbytes;
   mguard_t mg;
 
   /* Get the system page size and align break pointer so future sbrks will
@@ -839,6 +829,10 @@ internal_malloc (n, file, line, flags)		/* get a block */
      multiple of 8, then figure out which nextf[] area to use.  Try to
      be smart about where to start searching -- if the number of bytes
      needed is greater than the page size, we can start at pagebucket. */
+#if SIZEOF_SIZE_T == 8
+  if (ALLOCATED_BYTES(n) > MAXALLOC_SIZE)
+    return ((PTR_T) NULL);
+#endif
   nbytes = ALLOCATED_BYTES(n);
   nunits = (nbytes <= (pagesz >> 1)) ? STARTBUCK : pagebucket;
   for ( ; nunits < NBUCKETS; nunits++)
@@ -886,10 +880,8 @@ internal_malloc (n, file, line, flags)		/* get a block */
   p->mh_magic2 = MAGIC2;
   p->mh_nbytes = n;
 
-#if SIZEOF_CHAR_P == 8
   /* Begin guard */
-  MALLOC_MEMSET ((char *)p->mh_magic8, MAGIC1, 8);
-#endif
+  MALLOC_MEMSET ((char *)p->mh_magic8, MAGIC1, MAGIC8_NUMBYTES);
 
   /* End guard */
   mg.i = n;
@@ -937,16 +929,13 @@ internal_malloc (n, file, line, flags)		/* get a block */
 }
 
 static void
-internal_free (mem, file, line, flags)
-     PTR_T mem;
-     const char *file;
-     int line, flags;
+internal_free (PTR_T mem, const char *file, int line, int flags)
 {
   register union mhead *p;
   register char *ap, *z;
   register int nunits;
-  register unsigned int nbytes;
-  int ubytes;		/* caller-requested size */
+  register MALLOC_SIZE_T nbytes;
+  MALLOC_SIZE_T ubytes;		/* caller-requested size */
   mguard_t mg;
 
   if ((ap = (char *)mem) == 0)
@@ -979,28 +968,25 @@ internal_free (mem, file, line, flags)
 
   nunits = p->mh_index;
   nbytes = ALLOCATED_BYTES(p->mh_nbytes);
-  /* Since the sizeof(u_bits32_t) bytes before the memory handed to the user
-     are now used for the number of bytes allocated, a simple check of
-     mh_magic2 is no longer sufficient to catch things like p[-1] = 'x'.
+  /* The MAGIC8_NUMBYTES bytes before the memory handed to the user are now
+     used for a simple check to catch things like p[-1] = 'x'.
      We sanity-check the value of mh_nbytes against the size of the blocks
      in the appropriate bucket before we use it.  This can still cause problems
      and obscure errors if mh_nbytes is wrong but still within range; the
      checks against the size recorded at the end of the chunk will probably
-     fail then.  Using MALLOC_REGISTER will help here, since it saves the
+     fail then. Using MALLOC_REGISTER will help here, since it saves the
      original number of bytes requested. */
 
   if (IN_BUCKET(nbytes, nunits) == 0)
     xbotch (mem, ERR_UNDERFLOW,
 	    _("free: underflow detected; mh_nbytes out of range"), file, line);
-#if SIZEOF_CHAR_P == 8
   {
     int i;
-    for (i = 0, z = p->mh_magic8; i < 8; i++)
+    for (i = 0, z = p->mh_magic8; i < MAGIC8_NUMBYTES; i++)
       if (*z++ != MAGIC1)
 	xbotch (mem, ERR_UNDERFLOW,
 		_("free: underflow detected; magic8 corrupted"), file, line);
   }
-#endif
 
   ap += p->mh_nbytes;
   z = mg.s;
@@ -1011,19 +997,20 @@ internal_free (mem, file, line, flags)
 #if defined (USE_MMAP)
   if (nunits > malloc_mmap_threshold)
     {
+      int o;
+      o = errno;
       munmap (p, binsize (nunits));
 #if defined (MALLOC_STATS)
       _mstats.nlesscore[nunits]++;
 #endif
+      errno = o;		/* POSIX says free preserves errno */
       goto free_return;
     }
 #endif
 
-#if GLIBC21
-  if (nunits >= LESSCORE_MIN && ((char *)p + binsize(nunits) == sbrk (0)))
-#else
+#if defined (USE_LESSCORE)
+  /* We take care of the mmap case and munmap above */
   if (nunits >= LESSCORE_MIN && ((char *)p + binsize(nunits) == memtop))
-#endif
     {
       /* If above LESSCORE_FRC, give back unconditionally.  This should be set
 	 high enough to be infrequently encountered.  If between LESSCORE_MIN
@@ -1031,11 +1018,15 @@ internal_free (mem, file, line, flags)
 	 there's already a block on the free list. */
       if ((nunits >= LESSCORE_FRC) || busy[nunits] || nextf[nunits] != 0)
 	{
+	  int o;
+	  o = errno;
 	  lesscore (nunits);
+	  errno = o;
 	  /* keeps the tracing and registering code in one place */
 	  goto free_return;
 	}
     }
+#endif /* USE_LESSCORE */
 
 #ifdef MEMSCRAMBLE
   if (p->mh_nbytes)
@@ -1084,17 +1075,61 @@ free_return:
 #endif
 }
 
+#if USE_MREMAP == 1
+/* Assume the caller (internal_realloc) has already performed the sanity and
+   overflow tests. Basically we kill the old guard information, determine the
+   new size, call mremap with the new size, and add the bookkeeping and guard
+   information back in. */
 static PTR_T
-internal_realloc (mem, n, file, line, flags)
-     PTR_T mem;
-     register size_t n;
-     const char *file;
-     int line, flags;
+internal_remap (PTR_T mem, register size_t n, int nunits, int flags)
+{
+  register union mhead *p, *np;
+  char *m, *z;
+  mguard_t mg;
+  MALLOC_SIZE_T nbytes;
+
+  if (nunits >= NBUCKETS)	/* Uh oh */
+    return ((PTR_T) NULL);
+
+  p = (union mhead *)mem - 1;
+
+  m = (char *)mem + p->mh_nbytes;
+  z = mg.s;
+  *m++ = 0;  *m++ = 0;  *m++ = 0;  *m++ = 0;	/* erase guard */
+
+  nbytes = ALLOCATED_BYTES(n);
+
+  busy[nunits] = 1;
+  np = (union mhead *)mremap (p, binsize (p->mh_index), binsize (nunits), MREMAP_MAYMOVE);
+  busy[nunits] = 0;
+  if (np == MAP_FAILED)
+    return (PTR_T)NULL;
+
+  if (np != p)
+    {
+      np->mh_alloc = ISALLOC;
+      np->mh_magic2 = MAGIC2;
+      MALLOC_MEMSET ((char *)np->mh_magic8, MAGIC1, MAGIC8_NUMBYTES);
+    }
+  np->mh_index = nunits;
+  np->mh_nbytes = n;
+
+  mg.i = n;
+  z = mg.s;
+  m = (char *)(np + 1) + n;
+  *m++ = *z++, *m++ = *z++, *m++ = *z++, *m++ = *z++;
+
+  return ((PTR_T)(np + 1));
+}
+#endif
+
+static PTR_T
+internal_realloc (PTR_T mem, size_t n, const char *file, int line, int flags)
 {
   register union mhead *p;
-  register u_bits32_t tocopy;
-  register unsigned int nbytes;
-  register int nunits;
+  register MALLOC_SIZE_T tocopy;
+  register MALLOC_SIZE_T nbytes;
+  register int newunits, nunits;
   register char *m, *z;
   mguard_t mg;
 
@@ -1105,6 +1140,7 @@ internal_realloc (mem, n, file, line, flags)
   if (n == 0)
     {
       internal_free (mem, file, line, MALLOC_INTERNAL);
+      /* XXX - return internal_malloc (0, file, line MALLOC_INTERNAL) ? */
       return (NULL);
     }
   if ((p = (union mhead *) mem) == 0)
@@ -1132,16 +1168,14 @@ internal_realloc (mem, n, file, line, flags)
   if (IN_BUCKET(nbytes, nunits) == 0)
     xbotch (mem, ERR_UNDERFLOW,
 	    _("realloc: underflow detected; mh_nbytes out of range"), file, line);
-#if SIZEOF_CHAR_P == 8
   {
     int i;
-    for (i = 0, z = p->mh_magic8; i < 8; i++)
+    for (i = 0, z = p->mh_magic8; i < MAGIC8_NUMBYTES; i++)
       if (*z++ != MAGIC1)
 	xbotch (mem, ERR_UNDERFLOW,
 		_("realloc: underflow detected; magic8 corrupted"), file, line);
 
   }
-#endif
 
   m = (char *)mem + (tocopy = p->mh_nbytes);
   z = mg.s;
@@ -1161,11 +1195,15 @@ internal_realloc (mem, n, file, line, flags)
   if (n == p->mh_nbytes)
     return mem;
 
+#if SIZEOF_SIZE_T == 8
+  if (ALLOCATED_BYTES(n) > MAXALLOC_SIZE)
+    return ((PTR_T) NULL);
+#endif
   /* See if desired size rounds to same power of 2 as actual size. */
   nbytes = ALLOCATED_BYTES(n);
 
   /* If ok, use the same block, just marking its size as changed.  */
-  if (RIGHT_BUCKET(nbytes, nunits) || RIGHT_BUCKET(nbytes, nunits-1))
+  if (RIGHT_BUCKET(nbytes, nunits) || (nunits >= 1 && RIGHT_BUCKET(nbytes, nunits-1)))
     {
       /* Compensate for increment above. */
       m -= 4;
@@ -1187,12 +1225,31 @@ internal_realloc (mem, n, file, line, flags)
   _mstats.nrcopy++;
 #endif
 
-  /* If we are using mmap and have mremap, we could use it here. */
+#if USE_MREMAP == 1
+  /* If we are using mmap and have mremap, we use it here. Make sure that
+     the old size and new size are above the threshold where we use mmap */
+  if (nbytes > p->mh_nbytes)
+    newunits = nunits;
+  else
+    newunits = (nbytes <= (pagesz >> 1)) ? STARTBUCK : pagebucket;
+  for ( ; newunits < NBUCKETS; newunits++)
+    if (nbytes <= binsize(newunits))
+     break;
 
+  if (nunits > malloc_mmap_threshold && newunits > malloc_mmap_threshold)
+    {
+      m = internal_remap (mem, n, newunits, MALLOC_INTERNAL);
+      if (m == 0)
+        return 0;
+    }
+  else
+#endif /* USE_MREMAP */
+    {
   if ((m = internal_malloc (n, file, line, MALLOC_INTERNAL|MALLOC_NOTRACE|MALLOC_NOREG)) == 0)
     return 0;
   FASTCOPY (mem, m, tocopy);
   internal_free (mem, file, line, MALLOC_INTERNAL);
+    }
 
 #ifdef MALLOC_TRACE
   if (malloc_trace && (flags & MALLOC_NOTRACE) == 0)
@@ -1215,11 +1272,7 @@ internal_realloc (mem, n, file, line, flags)
 }
 
 static PTR_T
-internal_memalign (alignment, size, file, line, flags)
-     size_t alignment;
-     size_t size;
-     const char *file;
-     int line, flags;
+internal_memalign (size_t alignment, size_t size, const char *file, int line, int flags)
 {
   register char *ptr;
   register char *aligned;
@@ -1245,9 +1298,7 @@ internal_memalign (alignment, size, file, line, flags)
 }
 
 int
-posix_memalign (memptr, alignment, size)
-     void **memptr;
-     size_t alignment, size;
+posix_memalign (void **memptr, size_t alignment, size_t size)
 {
   void *mem;
 
@@ -1267,42 +1318,36 @@ posix_memalign (memptr, alignment, size)
 }
 
 size_t
-malloc_usable_size (mem)
-     void *mem;
+malloc_usable_size (void *mem)
 {
   register union mhead *p;
   register char *ap;
-  register int maxbytes;
-
 
   if ((ap = (char *)mem) == 0)
     return 0;
 
   /* Find the true start of the memory block to discover which bin */
   p = (union mhead *) ap - 1;
+
   if (p->mh_alloc == ISMEMALIGN)
     {
       ap -= p->mh_nbytes;
       p = (union mhead *) ap - 1;
     }
 
-  /* XXX - should we return 0 if ISFREE? */
-  maxbytes = binsize(p->mh_index);
-
-  /* So the usable size is the maximum number of bytes in the bin less the
-     malloc overhead */
-  maxbytes -= MOVERHEAD + MSLOP;
-  return (maxbytes);
+  /* return 0 if ISFREE */
+  if (p->mh_alloc == ISFREE)
+    return 0;
+  
+  /* Since we use bounds checking, the usable size is the last requested size. */
+  return (p->mh_nbytes);
 }
 
 #if !defined (NO_VALLOC)
 /* This runs into trouble with getpagesize on HPUX, and Multimax machines.
    Patching out seems cleaner than the ugly fix needed.  */
 static PTR_T
-internal_valloc (size, file, line, flags)
-     size_t size;
-     const char *file;
-     int line, flags;
+internal_valloc (size_t size, const char *file, int line, int flags)
 {
   return internal_memalign (getpagesize (), size, file, line, flags|MALLOC_INTERNAL);
 }
@@ -1310,10 +1355,7 @@ internal_valloc (size, file, line, flags)
 
 #ifndef NO_CALLOC
 static PTR_T
-internal_calloc (n, s, file, line, flags)
-     size_t n, s;
-     const char *file;
-     int line, flags;
+internal_calloc (size_t n, size_t s, const char *file, int line, int flags)
 {
   size_t total;
   PTR_T result;
@@ -1326,10 +1368,7 @@ internal_calloc (n, s, file, line, flags)
 }
 
 static void
-internal_cfree (p, file, line, flags)
-     PTR_T p;
-     const char *file;
-     int line, flags;
+internal_cfree (PTR_T p, const char *file, int line, int flags)
 {
   internal_free (p, file, line, flags|MALLOC_INTERNAL);
 }
@@ -1337,8 +1376,7 @@ internal_cfree (p, file, line, flags)
 
 #ifdef MALLOC_STATS
 int
-malloc_free_blocks (size)
-     int size;
+malloc_free_blocks (int size)
 {
   int nfree;
   register union mhead *p;
@@ -1353,58 +1391,38 @@ malloc_free_blocks (size)
 
 #if defined (MALLOC_WRAPFUNCS)
 PTR_T
-sh_malloc (bytes, file, line)
-     size_t bytes;
-     const char *file;
-     int line;
+sh_malloc (size_t bytes, const char *file, int line)
 {
   return internal_malloc (bytes, file, line, MALLOC_WRAPPER);
 }
 
 PTR_T
-sh_realloc (ptr, size, file, line)
-     PTR_T ptr;
-     size_t size;
-     const char *file;
-     int line;
+sh_realloc (PTR_T ptr, size_t size, const char *file, int line)
 {
   return internal_realloc (ptr, size, file, line, MALLOC_WRAPPER);
 }
 
 void
-sh_free (mem, file, line)
-     PTR_T mem;
-     const char *file;
-     int line;
+sh_free (PTR_T mem, const char *file, int line)
 {
   internal_free (mem, file, line, MALLOC_WRAPPER);
 }
 
 PTR_T
-sh_memalign (alignment, size, file, line)
-     size_t alignment;
-     size_t size;
-     const char *file;
-     int line;
+sh_memalign (size_t alignment, size_t size, const char *file, int line)
 {
   return internal_memalign (alignment, size, file, line, MALLOC_WRAPPER);
 }
 
 #ifndef NO_CALLOC
 PTR_T
-sh_calloc (n, s, file, line)
-     size_t n, s;
-     const char *file;
-     int line;
+sh_calloc (size_t n, size_t s, const char *file, int line)
 {
   return internal_calloc (n, s, file, line, MALLOC_WRAPPER);
 }
 
 void
-sh_cfree (mem, file, line)
-     PTR_T mem;
-     const char *file;
-     int line;
+sh_cfree (PTR_T mem, const char *file, int line)
 {
   internal_cfree (mem, file, line, MALLOC_WRAPPER);
 }
@@ -1412,10 +1430,7 @@ sh_cfree (mem, file, line)
 
 #ifndef NO_VALLOC
 PTR_T
-sh_valloc (size, file, line)
-     size_t size;
-     const char *file;
-     int line;
+sh_valloc (size_t size, const char *file, int line)
 {
   return internal_valloc (size, file, line, MALLOC_WRAPPER);
 }
@@ -1426,39 +1441,32 @@ sh_valloc (size, file, line)
 /* Externally-available functions that call their internal counterparts. */
 
 PTR_T
-malloc (size)
-     size_t size;
+malloc (size_t size)
 {
   return internal_malloc (size, (char *)NULL, 0, 0);
 }
 
 PTR_T
-realloc (mem, nbytes)
-     PTR_T mem;
-     size_t nbytes;
+realloc (PTR_T mem, size_t nbytes)
 {
   return internal_realloc (mem, nbytes, (char *)NULL, 0, 0);
 }
 
 void
-free (mem)
-     PTR_T mem;
+free (PTR_T mem)
 {
   internal_free (mem,  (char *)NULL, 0, 0);
 }
 
 PTR_T
-memalign (alignment, size)
-     size_t alignment;
-     size_t size;
+memalign (size_t alignment, size_t size)
 {
   return internal_memalign (alignment, size, (char *)NULL, 0, 0);
 }
 
 #ifndef NO_VALLOC
 PTR_T
-valloc (size)
-     size_t size;
+valloc (size_t size)
 {
   return internal_valloc (size, (char *)NULL, 0, 0);
 }
@@ -1466,15 +1474,13 @@ valloc (size)
 
 #ifndef NO_CALLOC
 PTR_T
-calloc (n, s)
-     size_t n, s;
+calloc (size_t n, size_t s)
 {
   return internal_calloc (n, s, (char *)NULL, 0, 0);
 }
 
 void
-cfree (mem)
-     PTR_T mem;
+cfree (PTR_T mem)
 {
   internal_cfree (mem, (char *)NULL, 0, 0);
 }
