@@ -1,6 +1,6 @@
 /* pathexp.h -- The shell interface to the globbing library. */
 
-/* Copyright (C) 1987-2020 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2023 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -49,7 +49,7 @@ extern int extended_glob;
 extern int glob_star;
 extern int match_ignore_case;	/* doesn't really belong here */
 
-extern int unquoted_glob_pattern_p PARAMS((char *));
+extern int unquoted_glob_pattern_p (char *);
 
 /* PATHNAME can contain characters prefixed by CTLESC; this indicates
    that the character is to be quoted.  We quote it here in the style
@@ -61,15 +61,17 @@ extern int unquoted_glob_pattern_p PARAMS((char *));
    pattern while executing a case statement), flags should include
    QGLOB_CVTNULL.  If flags includes QGLOB_FILENAME, appropriate quoting
    to match a filename should be performed. */
-extern char *quote_string_for_globbing PARAMS((const char *, int));
+extern char *quote_string_for_globbing (const char *, int);
 
-extern int glob_char_p PARAMS((const char *));
-extern char *quote_globbing_chars PARAMS((const char *));
+extern int glob_char_p (const char *);
+extern char *quote_globbing_chars (const char *);
 
 /* Call the glob library to do globbing on PATHNAME. FLAGS is additional
    flags to pass to QUOTE_STRING_FOR_GLOBBING, mostly having to do with
    whether or not we've already performed quote removal. */
-extern char **shell_glob_filename PARAMS((const char *, int));
+extern char **shell_glob_filename (const char *, int);
+
+extern char **noquote_glob_filename (char *);
 
 /* Filename completion ignore.  Used to implement the "fignore" facility of
    tcsh, GLOBIGNORE (like ksh-93 FIGNORE), and EXECIGNORE.
@@ -82,10 +84,11 @@ extern char **shell_glob_filename PARAMS((const char *, int));
 
 struct ign {
   char *val;
-  int len, flags;
+  size_t len;
+  int flags;
 };
 
-typedef int sh_iv_item_func_t PARAMS((struct ign *));
+typedef int sh_iv_item_func_t (struct ign *);
 
 struct ignorevar {
   char *varname;	/* FIGNORE, GLOBIGNORE, or EXECIGNORE */
@@ -95,10 +98,25 @@ struct ignorevar {
   sh_iv_item_func_t *item_func; /* Called when each item is parsed from $`varname' */
 };
 
-extern void setup_ignore_patterns PARAMS((struct ignorevar *));
+extern void setup_ignore_patterns (struct ignorevar *);
 
-extern void setup_glob_ignore PARAMS((char *));
-extern int should_ignore_glob_matches PARAMS((void));
-extern void ignore_glob_matches PARAMS((char **));
+extern void setup_glob_ignore (const char *);
+extern int should_ignore_glob_matches (void);
+extern void ignore_glob_matches (char **);
+
+/* Definitions for glob sorting */
+#define SORT_NONE	0
+#define SORT_NAME	1
+#define SORT_SIZE	2
+#define SORT_MTIME	3
+#define SORT_ATIME	4
+#define SORT_CTIME	5
+#define SORT_BLOCKS	6
+#define SORT_NUMERIC	7
+#define SORT_NOSORT	8
+
+#define SORT_REVERSE	128
+
+extern void setup_globsort (const char *);
 
 #endif
